#!/bin/bash

# It's necessar to use a subdirectory under /builds/pointbre as it's the only directory shared across all docker containers with docker in docker of gitlab ci/cd.
curr_dir=/builds/${CI_PROJECT_PATH}

###########################################
# The original notes and attachments directory
###########################################
org_notes_dir=${curr_dir}/notes
org_attachments_dir=${curr_dir}/attachments

###########################################
# Directories to store files for conversion
###########################################
# The collected notes and attachments directory to export to pdf and hugo
prepared_notes_dir=${curr_dir}/prepared/notes
prepared_attachments_dir=${curr_dir}/prepared/attachments
mkdir -p $prepared_notes_dir
mkdir -p $prepared_attachments_dir

###########################################
# Clone site git repo
###########################################
# - Clone git repo first
git config --global user.email "pointbre@gmail.com"
git config --global user.name "Lucas Kim"
git clone https://pointbre:$RWAPIAccessToken@gitlab.com/madforfamily/madforfamily.gitlab.io.git
# - Prepare directory variables
site_dir=${curr_dir}/madforfamily.gitlab.io
site_notes_dir=${site_dir}/content/post
site_attachments_dir=${site_dir}/content/attachments
site_exports_dir=${site_dir}/content/exports

###########################################
# Prepare hugo slugify
###########################################
# - Download hugo-slugify
git clone https://gitlab.com/pointbre/hugo-slugify.git
cp ./hugo-slugify/*.go .
# - Install Go modules first
go get golang.org/x/text/runes;
go get golang.org/x/text/transform; 
go get golang.org/x/text/unicode/norm;
# - Build slugify
go build hugo-slugify-link-in-file.go
go build hugo-slugify-string.go

###########################################
# Keep only markdown files with _MadForFamily.com_ tag
###########################################
while read -r file ; do
  echo "Copying $file to $prepared_notes_dir" 
  cp "$file" $prepared_notes_dir; 
done < <(grep -l "^tags:.*_MadForFamily.com_" $org_notes_dir/*.md)

###########################################
# Copy only files used by the copied markdown files
###########################################
for file in $org_attachments_dir/*
do
  file_name_only="`basename $file`"
  echo "Checking if $file_name_only is used"
  grep -q "$file_name_only" $prepared_notes_dir/*.md
  if [ $? -eq 0 ]; then
    echo "Copying $file to $prepared_attachments_dir" 
    cp "$file" $prepared_attachments_dir;     
  fi
done

###########################################
# Convert Notable's internal link to url of madforfamily.com for Pandoc and Hugo
###########################################
# Now it's possible to use Hugo's slug conversion logic, so the following sed command converting link to relref is no longer needed.
# sed -i -r 's/\]\(\.\/([^\)]*)\.md\)/]({{< relref "\1.md" >}})/g' $prepared_notes_dir/*.md
#
# Convert now
# Please note that you will see the warning message, "loadinternal: cannot find runtime/cgo", which doesn't affect conversion.
grep -Po -l "\]\(\.\/.*?\.md\)" $prepared_notes_dir/*.md | while read markdown_file;
do
  grep -Po "\]\(\.\/(.*?)\.md\)" "${markdown_file}" | while read original_link;
  do
    echo "Processing $original_link in $markdown_file"
    ${curr_dir}/hugo-slugify-link-in-file "${markdown_file}" "https://madforfamily.com/post/" "${original_link}"
  done
done

###########################################
# Convert Notable markdown files for Hugo
###########################################
# - Prepare directory variables
hugo_dir=${curr_dir}/hugo
hugo_notes_dir=${hugo_dir}/notes
hugo_attachments_dir=${hugo_dir}/attachments
hugo_exports_dir=${hugo_dir}/exports
mkdir -p $hugo_notes_dir
mkdir -p $hugo_attachments_dir
mkdir -p $hugo_exports_dir
# - Copy prepared files
rsync -av --delete ${prepared_notes_dir}/ ${hugo_notes_dir}/ --exclude .git/ --exclude .gitignore
rsync -av --delete ${prepared_attachments_dir}/ ${hugo_attachments_dir}/ --exclude .git/ --exclude .gitignore
# - Copy pdf files 
rsync -av --delete ${site_exports_dir}/ ${hugo_exports_dir}/ --exclude .git/ --exclude .gitignore
# - Converting working on Ubuntu
#   - Replace created: to date:, replace modified: to lastmod: and remove the tag _MadForFamily.com_
#     - The following works with gnu sed
#       sed -i 's/^created:/date:/g; s/^modified:/lastmod:/g; s/, _MadForFamily\.com_//g; s/_MadForFamily\.com_, //g' $prepared_notes_dir/*.md
#   - Remove lastmod in frontmatter as Hugo Academic doesn't use `date` when it exists
#     - The following works with gnu sed
#       sed -i '0,/^lastmod:/{/^lastmod:/d}' $prepared_notes_dir/*.md
#   - Remove the first header to prevent duplicate title displayed in rendered post
#     - The following works with gnu sed
#       sed -i '0,/^#/{/^#/d;}' $prepared_notes_dir/*.md
# - Converting working on Alpine Linux
for file in $hugo_notes_dir/*.md
do
  echo "Converting $file for Hugo"
  awk '!x{x=sub("created:","date:")}{print $0}' "$file" > /tmp/$$.tmp
  mv /tmp/$$.tmp "${file}"

  awk '!x{x=sub("modified:","lastmod:")}{print $0}' "$file" > /tmp/$$.tmp
  mv /tmp/$$.tmp "${file}"

  awk '!x{x=sub("_MadForFamily\.com_,","")}{print $0}' "$file" > /tmp/$$.tmp
  mv /tmp/$$.tmp "${file}"

  awk '!x{x=sub("_MadForFamily\.com_","")}{print $0}' "$file" > /tmp/$$.tmp
  mv /tmp/$$.tmp "${file}"

  awk '/^lastmod:/ && ++c==1{next}1' "$file" > /tmp/$$.tmp
  mv /tmp/$$.tmp "${file}"

  awk '/^#/ && ++c==1{next}1' "$file" > /tmp/$$.tmp
  mv /tmp/$$.tmp "${file}"
done

###########################################
# Convert Notable markdown files to pdf format
###########################################
# - Copy necessary files
cp $org_notes_dir/.pdf.tex $prepared_notes_dir
cp $org_notes_dir/.puppeteer.json $prepared_notes_dir
cp $org_notes_dir/.mermaid-config.json $prepared_notes_dir
cp $org_notes_dir/.mermaid.css $prepared_notes_dir
# - Go to notes directory as we need to above files which must be located in working directory
cd $prepared_notes_dir
# - Convert now
for file in *.md
do
  name_only=$(basename "$file" .md)
  slugified_name_only=$(${curr_dir}/hugo-slugify-string "${name_only}")
  post_url="https://madforfamily.com/post/${slugified_name_only}.html"

  echo "Checking if pdf for $file needs to be converted"
  need_to_convert_to_pdf="N"
  if [ ! -e "${hugo_exports_dir}/${slugified_name_only}.pdf" ]; then
    need_to_convert_to_pdf="Y"
  else
    diff -q "$hugo_notes_dir/$file" "$site_notes_dir/$file" > /dev/null
    if [ $? -ne 0 ] ; then
      need_to_convert_to_pdf="Y"
    fi
  fi

  echo "Converting required? $need_to_convert_to_pdf"
  if [ "$need_to_convert_to_pdf" = "Y" ] ; then
    echo "Converting $file to pdf";
    sed -i "1{/^---$/!q;};1,/^---$/d" "${name_only}.md"
    # Set post url as pdf title in pdf properties and print in header
    sed -i "/pdftitle/c    pdftitle={${post_url}}" .pdf.tex
    sed -i "/\\\rhead/c \\\\\rhead{${post_url}}" .pdf.tex
    pandoc "${name_only}.md" \
      -f gfm-raw_tex \
      --pdf-engine=xelatex \
      -V mainfont=NanumMyeongjo \
      -V sansfond=NanumGothic \
      -V monofont=NanumGothic \
      -H .pdf.tex \
      -V colorlinks \
      -V urlcolor=NavyBlue \
      -V papersize:a4 \
      -V geometry=margin=2cm \
      --toc \
      -V toc-title="Table of contents" \
      -N \
      -V subparagraph \
      -F mermaid-filter \
      -o "${hugo_exports_dir}/${slugified_name_only}.pdf"
  else
    echo "Not converting $file to pdf because it's not changed"
  fi
done
# - Go back to current directory
cd $curr_dir

###########################################
# Apply changes to madforfamily.gitlab.io
###########################################
echo "Syncing ${hugo_notes_dir}/ to ${site_notes_dir}/"
rsync -av --delete ${hugo_notes_dir}/ ${site_notes_dir}/ --exclude .git/ --exclude .gitignore
echo "Syncing ${hugo_attachments_dir}/ ${site_attachments_dir}/"
rsync -av --delete ${hugo_attachments_dir}/ ${site_attachments_dir}/ --exclude .git/ --exclude .gitignore
echo "Syncing ${hugo_exports_dir}/ ${site_exports_dir}/"
rsync -av --delete ${hugo_exports_dir}/ ${site_exports_dir}/ --exclude .git/ --exclude .gitignore

###########################################
# Push changes
###########################################
cd $site_dir
git add . || true
git commit -m "Synchronized with notes" || true
git push || true
